#include "Dut.h"

	
Dut::Dut(SpvModule *owner,string name):
    SpvModule(owner,name),      //Set name and link to parent module
                                //Name signals
    m_Clk(FullPath() + ".clock"),  
    m_ResetN(FullPath() + ".reset_n"),  
    m_DataIn(FullPath() + ".data_in"),  
    m_DataInValid(FullPath() + ".data_in_valid")  ,
    m_DataOut(FullPath() + ".data_out"),  
    m_DataOutValid(FullPath() + ".data_out_valid")  
{
        m_FifoDepth = 16;
};

void Dut::OnInitSim()
{
    //Set size of signals
	SpvNullSim::SetSigSize(m_Clk, 1);
	SpvNullSim::SetSigSize(m_ResetN, 1);
	SpvNullSim::SetSigSize(m_DataIn, 4);
	SpvNullSim::SetSigSize(m_DataInValid, 1);
    SpvNullSim::SetSigSize(m_DataOut, 4);
	SpvNullSim::SetSigSize(m_DataOutValid, 1);
}

void Dut::OnInitial() 
{
    m_DataOutValid = 0;
    m_DataOut      = 0;
}

void Dut::InitConnection(SpvSig& clk, 
                         SpvSig& resetN, 
                         SpvSig& dataIn, 
                         SpvSig& dataInValid, 
                         SpvWireConnection& dataOut, 
                         SpvWireConnection& dataOutValid) 
{ 
    m_Clk.Init(clk); 
    m_ResetN.Init(resetN); 
    m_DataIn.Init(dataIn); 
    m_DataInValid.Init(dataInValid); 
    dataOut.Init(dataOut); 
    dataOutValid.Init(dataOutValid); 
    //Start atomic process on positive edge of clock
    StartAProc(m_Clk, AtPos, this, (SPVPM)&Dut::PosClkFunc);

    //Start atomic process on negative edge of reset
    StartAProc(m_ResetN, AtNeg, this, (SPVPM)&Dut::NegResetFunc);
} 

void Dut::NegResetFunc() 
{
    m_DataFifo.clear();
    m_DataEnFifo.clear();
}

void Dut::PosClkFunc() 
{
    if(m_ResetN != 0)
    {
        unsigned inData = m_DataIn.Uint();
        m_DataFifo.push_back(m_DataIn.Uint());
        m_DataEnFifo.push_back(m_DataInValid.Uint());

        if(m_DataFifo.size() >= m_FifoDepth)
        {
            unsigned outData = ~m_DataFifo.front() & 0xF;
            m_DataOut       = ~m_DataFifo.front() & 0xF;
            m_DataFifo.pop_front();

            m_DataOutValid  = m_DataEnFifo.front();
            m_DataEnFifo.pop_front();
        }
    }
}
