#ifndef TOP
#define TOP
#include "SpvHfile.h"
#include "SpvNullSim.h"

//Submodule definition
class	Channel : public SpvModule
{
	friend class top;
	SpvWireConnection	m_Clk;
	
public:
    Channel(SpvModule *owner,string name):
        SpvModule(owner,name),      //Set name and link to parent module
        m_Clk(FullPath() + ".clk")  //Name clock wire
    {
    };

	void OnInitSim()
    {
	    SpvNullSim::SetSigSize(m_Clk, 1);

        //Start atomic process on negative edge of clock
        StartAProc(m_Clk, AtNeg, this, (SPVPM)&Channel::NegClkFunc);
    }

    void OnInitial() {}

    //Connect outer module's clk sig to m_Clk wire
    void InitConnection(SpvSig &clk) { m_Clk.Init(clk); } 


    //Atomic process on negative edge of clock
	void NegClkFunc() 
    {
        //Place clock process code here
    }
};

//top level module
class top : 
    public SpvModule, 
    public SpvSimDeltaT
{
	SpvSig	    m_Clk;
	Channel*    m_Channel0;
	Channel*    m_Channel1;
public:

	top(SpvModule* owner,string name):
	    SpvModule(owner,name),      //Set name and link to parent module
	    m_Clk(FullPath() + ".clk"), //Name clock sig  
	    SpvSimDeltaT(50)            //Setup call to SpvOnDelta() every 50 units
    {
	    m_Channel0 = NULL;
	    m_Channel1 = NULL;
    }
	
    void SpvOnDelta() { m_Clk = ~m_Clk; }
	
	void OnInitSim()
    {
        //Set size of clock sig
	    SpvNullSim::SetSigSize(m_Clk, 1);

        //Instantiate submodules
        m_Channel0 = new Channel(this,"channel0");
	    m_Channel1 = new Channel(this,"channel1");

        //Connect submodules to clock sig
	    m_Channel0->InitConnection(m_Clk);
	    m_Channel1->InitConnection(m_Clk);
    }

    //Set initial signal values
	void OnInitial() { m_Clk = 0; }
};


#endif
